@@ -34,6 +34,7 @@ PERFORMANCE OF THIS SOFTWARE.
 #define my_bool _Bool
 #endif
 
+#define PY_SSIZE_T_CLEAN 1
 #include "Python.h"
 #if PY_MAJOR_VERSION >= 3
 #define IS_PY3K
@@ -216,6 +217,9 @@ _get_encoding(MYSQL *mysql)
     if (strncmp(utf8, cs.csname, 4) == 0) { // utf8, utf8mb3, utf8mb4
         return utf8;
     }
+    else if (strncmp("latin1", cs.csname, 6) == 0) {
+        return "cp1252";
+    }
     else if (strncmp("koi8r", cs.csname, 5) == 0) {
         return "koi8_r";
     }
@@ -396,7 +400,8 @@ _mysql_ConnectionObject_Initialize(
                   "read_default_file", "read_default_group",
                   "client_flag", "ssl",
                   "local_infile",
-                  "read_timeout", "write_timeout",
+                  "read_timeout", "write_timeout", "charset",
+                  "auth_plugin",
                   NULL } ;
     int connect_timeout = 0;
     int read_timeout = 0;
@@ -404,13 +409,15 @@ _mysql_ConnectionObject_Initialize(
     int compress = -1, named_pipe = -1, local_infile = -1;
     char *init_command=NULL,
          *read_default_file=NULL,
-         *read_default_group=NULL;
+         *read_default_group=NULL,
+         *charset=NULL,
+         *auth_plugin=NULL;
 
     self->converter = NULL;
     self->open = 0;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
-                "|ssssisOiiisssiOiii:connect",
+                "|ssssisOiiisssiOiiiss:connect",
                 kwlist,
                 &host, &user, &passwd, &db,
                 &port, &unix_socket, &conv,
@@ -421,7 +428,9 @@ _mysql_ConnectionObject_Initialize(
                 &client_flag, &ssl,
                 &local_infile,
                 &read_timeout,
-                &write_timeout
+                &write_timeout,
+                &charset,
+                &auth_plugin
     ))
         return -1;
 
@@ -444,8 +453,12 @@ _mysql_ConnectionObject_Initialize(
         _stringsuck(cipher, value, ssl);
     }
 
-    Py_BEGIN_ALLOW_THREADS ;
     conn = mysql_init(&(self->connection));
+    if (!conn) {
+        PyErr_SetNone(PyExc_MemoryError);
+        return -1;
+    }
+    Py_BEGIN_ALLOW_THREADS ;
     self->open = 1;
     if (connect_timeout) {
         unsigned int timeout = connect_timeout;
@@ -481,6 +494,12 @@ _mysql_ConnectionObject_Initialize(
     if (ssl) {
         mysql_ssl_set(&(self->connection), key, cert, ca, capath, cipher);
     }
+    if (charset) {
+        mysql_options(&(self->connection), MYSQL_SET_CHARSET_NAME, charset);
+    }
+    if (auth_plugin) {
+        mysql_options(&(self->connection), MYSQL_DEFAULT_AUTH, auth_plugin);
+    }
 
     conn = mysql_real_connect(&(self->connection), host, user, passwd, db,
                   port, unix_socket, client_flag);
@@ -497,7 +516,6 @@ _mysql_ConnectionObject_Initialize(
 
     if (!conn) {
         _mysql_Exception(self);
-        self->open = 0;
         return -1;
     }
 
@@ -732,7 +750,7 @@ _mysql_ConnectionObject_commit(
 }
 
 static char _mysql_ConnectionObject_rollback__doc__[] =
-"Rolls backs the current transaction\n\
+"Rolls back the current transaction\n\
 ";
 static PyObject *
 _mysql_ConnectionObject_rollback(
@@ -879,7 +897,8 @@ _mysql_escape_string(
 {
     PyObject *str;
     char *in, *out;
-    int len, size;
+    int len;
+    Py_ssize_t size;
     if (!PyArg_ParseTuple(args, "s#:escape_string", &in, &size)) return NULL;
     str = PyBytes_FromStringAndSize((char *) NULL, size*2+1);
     if (!str) return PyErr_NoMemory();
@@ -1099,7 +1118,7 @@ static PyObject *
 _mysql_field_to_python(
     PyObject *converter,
     const char *rowitem,
-    unsigned long length,
+    Py_ssize_t length,
     MYSQL_FIELD *field,
     const char *encoding)
 {
@@ -1147,16 +1166,17 @@ _mysql_field_to_python(
 #endif
         // Call converter with bytes
         binary = 1;
+        break;
     default: // e.g. FIELD_TYPE_DATETIME, etc.
         // Call converter with unicode string
         binary = 0;
     }
     return PyObject_CallFunction(converter,
             binary ? "y#" : "s#",
-            rowitem, (int)length);
+            rowitem, (Py_ssize_t)length);
 #else
     return PyObject_CallFunction(converter,
-            "s#", rowitem, (int)length);
+            "s#", rowitem, (Py_ssize_t)length);
 #endif
 }
 
@@ -1748,7 +1768,8 @@ _mysql_ConnectionObject_query(
     PyObject *args)
 {
     char *query;
-    int len, r;
+    Py_ssize_t len;
+    int r;
     if (!PyArg_ParseTuple(args, "s#:query", &query, &len)) return NULL;
     check_connection(self);
 
@@ -1770,7 +1791,8 @@ _mysql_ConnectionObject_send_query(
     PyObject *args)
 {
     char *query;
-    int len, r;
+    Py_ssize_t len;
+    int r;
     MYSQL *mysql = &(self->connection);
     if (!PyArg_ParseTuple(args, "s#:query", &query, &len)) return NULL;
     check_connection(self);
@@ -2283,7 +2305,7 @@ static struct PyMemberDef _mysql_ConnectionObject_memberlist[] = {
     },
     {
         "server_capabilities",
-        T_UINT,
+        T_ULONG,
         offsetof(_mysql_ConnectionObject,connection.server_capabilities),
         READONLY,
         "Capabilities of server; consult MySQLdb.constants.CLIENT"
@@ -2297,7 +2319,7 @@ static struct PyMemberDef _mysql_ConnectionObject_memberlist[] = {
     },
     {
         "client_flag",
-        T_UINT,
+        T_ULONG,
         offsetof(_mysql_ConnectionObject,connection.client_flag),
         READONLY,
         "Client flags; refer to MySQLdb.constants.CLIENT"
@@ -2635,6 +2657,15 @@ init_mysql(void)
 {
     PyObject *dict, *module, *emod, *edict;
 
+    if (mysql_library_init(0, NULL, NULL)) {
+        PyErr_SetString(PyExc_ImportError, "_mysql: mysql_library_init failed");
+#ifdef IS_PY3K
+        return NULL;
+#else
+        return;
+#endif
+    }
+
 #ifdef IS_PY3K
     if (PyType_Ready(&_mysql_ConnectionObject_Type) < 0)
         return NULL;
